import { select } from 'd3';
import { lineChart } from './lineChart';
import { scatterPlot } from './scatterPlot';
import { yearlyData } from '@python-monty/full_cyber_attack_data';
import { newData } from '@python-monty/full_cyber_attack_data';
import { observeResize } from '@curran/responsive-axes';
import { menu } from './menu';
import './styles.css';

// <LINE VIZ> PREPARE THE MENU OPTIONS
const menuOptions = [
  { value: '11', text: 'Agriculture' },
  { value: '21', text: 'Natural Resoure Extraction' },
  { value: '22', text: 'Utilities' },
  { value: '23', text: 'Construction' },
  { value: '33', text: 'Manufacturing' },
  { value: '42', text: 'Wholesale Trade' },
  { value: '44', text: 'Retail Trade' },
  { value: '48', text: 'Trans & Warehouse' },
  { value: '51', text: 'Information' },
  { value: '52', text: 'Finance & Insurance' },
  { value: '53', text: 'Real Estate' },
  { value: '54', text: 'Pro, Sci, Tech Services' },
  { value: '55', text: 'Enterprise Management' },
  { value: '56', text: 'Waste Management' },
  { value: '61', text: 'Education' },
  { value: '62', text: 'Heath Care' },
  { value: '71', text: 'Arts, Entertain, Recreation' },
  { value: '72', text: 'Food Services' },
  { value: '81', text: 'Other Services' },
  { value: '92', text: 'Public Administration' },
];

// INDUSTRY LOOKUP TABLE
const industryLookup = {
  11: 'Agriculture',
  21: 'Natural Resoure Extraction',
  22: 'Utilities',
  23: 'Construction',
  33: 'Manufacturing',
  42: 'Wholesale Trade',
  44: 'Retail Trade',
  48: 'Trans & Warehouse',
  51: 'Information',
  52: 'Finance & Insurance',
  53: 'Real Estate',
  54: 'Pro, Sci, Tech Services',
  55: 'Enterprise Management',
  56: 'Waste Management',
  61: 'Education',
  62: 'Heath Care',
  71: 'Arts, Entertain, Recreation',
  72: 'Food Services',
  81: 'Other Services',
  92: 'Public Administration',
};

// // <SCATTERPLOT> FILTER THE DATA
// let filter = parseInt(33); //  <---- for instance   11, this will rec input from the other viz (eventually)

// const filters = new Set([filter]);

// // <SCATTERPLOT> FILTER OUT ALL OTHER DATA THAT DOES NOT HAVE AN INDUSTRY CODE OF 11,21 OR 43
// const filteredScatterData = newData.filter(
//   ({ industry_code }) => filters.has(industry_code),
// );

// // <SCATTERPLOT> SLICE ALL EVENT SUBTYPE STRINGS DOWN TO ONLY 10 DIGITS (FOR EASIER RENDERING ON SCREEN)
// filteredScatterData.forEach(function (part, index) {
//   filteredScatterData[index].event_subtype =
//     filteredScatterData[index].event_subtype.slice(0, 10);

//   filteredScatterData[index].actor = filteredScatterData[
//     index
//   ].actor.slice(0, 10);
// });

export const main = (container, { state, setState }) => {
  // Initialize the selected menu options.
  if (state.menu1Value === undefined) {
    setState((state) => ({
      ...state,
      menu1Value: menuOptions[0].value,
      menu2Value: menuOptions[1].value,
      menu3Value: menuOptions[2].value,
    }));
    return;
  }

  const dimensions = observeResize({
    state,
    setState,
    container,
  });

  if (dimensions === null) return;
  let { width, height } = dimensions;
  height = height - 100;

  const containerSelection = select(container).attr(
    'class',
    'ui-elements-container',
  );

  const menusContainer = containerSelection
    .selectAll('div.menus-container')
    .data([null])
    .join('div')
    .attr('class', 'menus-container');

  menusContainer
    .selectAll('.menu')
    .data([
      { label: 'stuff', stateField: 'menu1Value' },
      { label: '', stateField: 'menu2Value' },
      { label: '', stateField: 'menu3Value' },
    ])
    .join((enter) =>
      enter
        .append('div')
        .attr('class', 'menu d-flex')
        .html(
          (d) => `
            <label class="col-form-label">${d.label}</label>
            <div class="menu-container ms-3"></div>
          `,
        ),
    )
    .each(function ({ label, stateField }) {
      select(this).select('div.menu-container').call(menu, {
        state,
        setState,
        stateField,
        menuOptions,
      });
    });

  // Display the count as text
  const {
    menu1Value,
    menu2Value,
    menu3Value,
    hoveredValue,
  } = state;

  console.log("HERE's the state of hoveredValue:");
  console.log(hoveredValue);

  // <SCATTERPLOT> FILTER THE DATA. CONTROLS WHAT IS SHOWN IN THE SCATTERPLOT VIZ ACCORDING TO WHAT LINE VIZ COLOR LEGEND ITEM IS HOVERED OVER
  let filter = parseInt(hoveredValue);

  const scatterFilters = new Set([filter]);

  // <SCATTERPLOT> FILTER OUT ALL OTHER DATA THAT DOES NOT HAVE AN INDUSTRY CODE OF 11,21 OR 43
  const filteredScatterData = newData.filter(
    ({ industry_code }) =>
      scatterFilters.has(industry_code),
  );

  // <SCATTERPLOT> SLICE ALL EVENT SUBTYPE STRINGS DOWN TO ONLY 10 DIGITS (FOR EASIER RENDERING ON SCREEN)
  filteredScatterData.forEach(function (part, index) {
    filteredScatterData[index].event_subtype =
      filteredScatterData[index].event_subtype.slice(0, 10);

    filteredScatterData[index].actor = filteredScatterData[
      index
    ].actor.slice(0, 10);
  });

  // <LINE VIZ> FILTER OUT DATA OTHER THAN WHAT THE USER SELECTED
  let filter1 = parseInt(menu1Value); //  <---- for instance   11
  let filter2 = parseInt(menu2Value); //  <---- for instance   21
  let filter3 = parseInt(menu3Value); //  <---- for instance   43

  const filters = new Set([filter1, filter2, filter3]);

  // <LINE VIZ> FILTER OUT ALL OTHER DATA THAT DOES NOT HAVE AN INDUSTRY CODE OF 11,21 OR 43
  const filteredLineData = yearlyData.filter(
    ({ industryCode }) => filters.has(industryCode),
  );

  const svg = select(container)
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width)
    .attr('height', height);

  const setHoveredValue = (d) => {
    setState((state) => ({
      ...state,
      hoveredValue: d,
    }));
  };

  svg
    .selectAll('g.multi-line-chart')
    .data([null])
    .join('g')
    .attr('class', 'multi-line-chart')
    // svg.selectAll('g.bar-chart');
    .call(lineChart, {
      filteredLineData,
      xValue: (d) => d.UTCyear,
      xAxisLabelText: 'Year',
      xAxisLabelOffset: 30,
      yValue: (d) => d.count,
      lineValue: (d) => d.industryCode,
      yAxisLabelText: 'Number of Attacks',
      yAxisLabelOffset: 80,
      innerRectFill: '#E8E8E8',
      // circleRadius: 35 / 20,
      // circleOpacity: 629 / 1000,
      marginTop: 20,
      marginBottom: 60,
      marginLeft: 120,
      marginRight: 20,
      width: width / 2,
      height,
      colorLegendLabel: 'Industries',
      colorLegendX: 20,
      colorLegendY: 50,
      setHoveredValue,
    });
  console.log(filteredLineData);

  svg
    .selectAll('g.scatter-chart')
    .data([null])
    .join('g')
    .attr('class', 'scatter-chart')
    .attr('transform', `translate(${width / 2},0)`)
    .call(scatterPlot, {
      filteredScatterData,
      width: width / 2,
      height,
      xValue: (d) => d.year,
      yValue: (d) => d.event_subtype,
      colorValue: (d) => d.actor,
      index: (d) => d.key,
      xAxisLabel: 'Event Date',
      yAxisLabel: 'Attack Type',
      colorLegendLabel: 'Actor',
      margin: {
        top: 20,
        right: 150,
        bottom: 60,
        left: 100,
      },
      colorLegendX: width / 2 - 100,
      colorLegendY: height - 441,
      state,
      setState,
      hoveredValue,
    });
  console.log(filteredScatterData);
};
