import {
  scaleLinear,
  scaleTime,
  scaleOrdinal,
  axisLeft,
  extent,
  line,
  flatGroup,
  schemeCategory10,
} from 'd3';
import {
  xAxis,
  yAxis,
  xAxisLabel,
  yAxisLabel,
} from '@python-monty/corey-responsive-axis';
import { colorLegend } from './colorLegend';

export const lineChart = (
  svg,
  {
    filteredLineData,
    xValue,
    xAxisLabelText,
    xAxisLabelOffset,
    yValue,
    lineValue,
    yAxisLabelText,
    yAxisLabelOffset,
    marginLeft,
    marginTop,
    marginRight,
    marginBottom,
    width,
    height,
    innerRectFill,
    // circleRadius,
    // circleOpacity,
    colorLegendX,
    colorLegendY,
    colorLegendLabel,
    setHoveredValue,
    hoveredValue,
    // menuValuesArray,
  },
) => {
  const xScale = scaleTime(extent(filteredLineData, xValue), [
    marginLeft,
    width - marginRight,
  ]);

  const yScale = scaleLinear(extent(filteredLineData, yValue), [
    height - marginBottom,
    marginTop,
  ]);

  const altColorScale = scaleOrdinal()
    .domain(filteredLineData.map(lineValue))
    .range(schemeCategory10);

  const innerWidth = width - marginLeft - marginRight;
  const innerHeight = height - marginTop - marginBottom;

  svg
    .selectAll('rect')
    .data([null])
    .join('rect')
    .attr('x', marginLeft)
    .attr('y', marginTop)
    .attr('width', innerWidth)
    .attr('height', innerHeight)
    .attr('fill', innerRectFill);

  // const [xMin, xMax] = xScale.range();
  // const [yMax, yMin] = yScale.range();

  // Axes
  xAxis(svg, { height, marginBottom, xScale });
  yAxis(svg, { marginLeft, yScale });

  // Axis Labels
  xAxisLabel(svg, {
    xAxisLabelText,
    width,
    height,
    marginBottom,
    xAxisLabelOffset,
  });
  yAxisLabel(svg, {
    yAxisLabelText,
    yAxisLabelOffset,
    height,
  });

  const lineGenerator = line(
    (d) => xScale(xValue(d)),
    (d) => yScale(yValue(d)),
  ).defined(yValue);

  const dataGrouped = flatGroup(filteredLineData, (d) => {
    return d.industryCode;
  });

  svg.call(colorLegend, {
    altColorScale,
    colorLegendLabel,
    colorLegendX,
    colorLegendY,
    setHoveredValue,
    hoveredValue,
  });

  svg
    .selectAll('path.mark')
    .data(dataGrouped)
    .join('path')
    .attr('class', 'mark')
    .attr('fill', 'none')
    .attr('stroke', (d) => {
      return altColorScale(d[0]);
    })
    .style('stroke-width', 5)
    .attr('opacity', 0.5)
    .attr('d', (d) => lineGenerator(d[1]));
};
